SystemOrganization addCategory: #ExtentionsForKendrick!

TestCase subclass: #KEReproducibleSimulatorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!KEReproducibleSimulatorTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/13/2016 16:40'!
model

	|model|

		model := KEModel new population: (KEPopulation size: 100000).
	model attributes: {#status->#(#S #I #R)}.
	model atCompartment: { #status->#S } put: 99999 atOthersPut: 0.
	model atCompartment: { #status->#I } put: 1.
	model addParameters: {#beta->520. #gamma->52}.
	model addTransitionFrom: '{#status: #S}'
		to: '{#status: #I}'
		probability: [ :m| 
			(m atParameter: #beta) * 
			(m atCompartment: {#status->#I}) / (m atParameter: #N)].
	model addTransitionFrom: '{#status: #I}'
		to: '{#status: #R}'
		probability: [ :m| (model atParameter: #gamma)].
	
	^ model! !

!KEReproducibleSimulatorTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/13/2016 17:23'!
testAfterSimulationOfSIRModelThereAreThreeTimeSeries

| simulator1 simulator2 |
	
	simulator1 := KESimulator new: #Gillespie from: 0.0 to: 1.0 step: 0.001.
	simulator2 := KESimulator new: #Gillespie from: 0.0 to: 1.0 step: 0.001.
	simulator1 executeOn: self model.
	simulator2 executeOn: self model.
	self assert: simulator1 allTimeSeries size = 3.
	
	Transcript cr; cr; show: '===gillespie==='; cr; cr.
	
	1 to: 3 do: [ :i | "Vanilla Gillespie is stochastic"
		Transcript 
			show: (simulator1 allTimeSeries at: i) data;
			show: ' =? ';
			show: (simulator2 allTimeSeries at: i) data;
			cr.
		self assert: ((simulator1 allTimeSeries at: i) data  = (simulator2 allTimeSeries at: i) data) not].
	
	Transcript cr; cr; show: '===reproducible-gillespie==='; cr; cr.
	
	KEGillespieSimulatorForTesting newSeed. "random seed but constant between calls to new seed"
	simulator1 := (KEGillespieSimulatorForTesting new) tMin: 0.0; tMax: 1.0; step: 0.001.
	simulator2 := (KEGillespieSimulatorForTesting new) tMin: 0.0; tMax: 1.0; step: 0.001.
	simulator1 executeOn: self model.
	simulator2 executeOn: self model.
	self assert: simulator1 allTimeSeries size = 3.
	1 to: 3 do: [ :i | "Test Gillespie is reproducibly stochastic"
		Transcript 
			show: (simulator1 allTimeSeries at: i) data;
			show: ' =? ';
			show: (simulator2 allTimeSeries at: i) data;
			cr.
		self assert: ((simulator1 allTimeSeries at: i) data  = (simulator2 allTimeSeries at: i) data)].! !

TestCase subclass: #TestMeaslesScripts
	instanceVariableNames: 'beforeSimulator afterSimulator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 18:52'!
after

Model	Measles
	attribute: #(status -> S E I R);
	parameters: #(beta gamma mu sigma); 
	equations: #(
	  S:t=mu*N - beta*S*I - mu*S.
      E:t=beta*S*I - sigma*E - mu*E.
      I:t=sigma*E - gamma*I - mu*I.
      R:t=gamma*I - mu*R.
	);
	population: 100000;
	S: 99999;
	I: 1;
	others: 0;
	beta: 0.0000214;
	gamma: 0.143;
	mu: 0.0000351;
	sigma: 0.125.	
		
Simulation MeaslesRKSim rungeKutta
	for: 'Measles';
	from: 0.0; 
	to: 150; 
	step: 1.
	
Visualization MeaslesDiagramViz diagram 
	for: 'MeaslesRKSim';
	xLabel: 'Time (days)'.! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:32'!
before

| model simulator diag |
model := KEModel new population: (KEPopulation size: 100000).
model addAttribute: #status value: #(S E I R).
model atCompartment: { #status->#S } put: 99999 atOthersPut: 0.
model atCompartment: { #status->#I } put: 1.
model addEquations: { 
            'S:t=mu*N - beta*S*I - mu*S'.
            'E:t=beta*S*I - sigma*E - mu*E'.
            'I:t=sigma*E - gamma*I - mu*I'.
            'R:t=gamma*I - mu*R'
            }.
model addParameters: {
        #beta->0.0000214. 
        #gamma->0.143. 
        #mu->0.0000351. 
        #sigma->0.125 }.
simulator := KESimulator new: #RungeKutta from: 0.0 to: 150 step: 1.
simulator executeOn: model.
diag := (KEDiagramBuilder new) data: simulator allTimeSeries.
diag xLabel: 'Time (days)'.
diag open.

^ simulator! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:38'!
setUp

	beforeSimulator := self before.
	self after. "we grab references by id to show-case how tools should handle the declarative dsl"
	afterSimulator := Simulation withId: 'MeaslesRKSim'.
	(Visualization withId: 'MeaslesDiagramViz') open.! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:34'!
tearDown

	KendrickInternalDSLEntities clearAll.! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:48'!
testEquivalence

	self assert: 
		(beforeSimulator allTimeSeries collect: [ :ts | ts peakOfEpidemic  ]) = 
		(afterSimulator allTimeSeries collect: [ :ts | ts peakOfEpidemic  ])! !

TestCase subclass: #TestMosquitoScripts
	instanceVariableNames: 'beforeSimulator afterSimulator beforePair beforeModel afterModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestMosquitoScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 19:18'!
after

Model Mosquito
	population: 13000;
	attribute: #(status -> S I R); 
	parameters: #(mu gamma beta lambda rho);
	mu: 12.17;
	gamma: 52;
	beta: 1;
	lambda: #(beta*I);
	transitions: #(
		S -- lambda --> I.
		I -- gamma --> R.
		status -- mu --> Empty.
		Empty -- mu --> S.
	).

Concern SpeciesConcern
	attribute: #(species -> mosquito reservoir1 reservoir2).
		
Composition MultiMosquito
	model: 'Mosquito';
	concern: 'SpeciesConcern';	
	S_species: #(9999 1000 2000);
	I_species: #(1 0 0);
	"others: 0;"
	N: #(species);
	mu_species: #(12.17 0.05 0.05);
	rho_species: #(
		#(0 0.02 0.02) 
		#(0.02 0 0) 
		#(0.02 0 0)
	);
	lambda: #(beta*rho*I sum).

Simulation MosquitoGillepsie gillespie
	for: 'Mosquito';
	from: 0.0; 
	to: 0.5; 
	step: 0.0027.
	
Visualization MosquitoDiagramViz diagram 
	for: 'MosquitoGillepsie';
	data: #(I sqrt);
	xLabel: 'Time (days)'.! !

!TestMosquitoScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/12/2016 19:09'!
before

| model multiHostConcern simulator db |
model := KEModel new.
model population: (KEPopulation size: 13000).
model attributes: {(#status -> #(#S #I #R))}.
model addParameter: #mu value: 12.17.
model addParameter: #gamma value: 52.
model addParameter: #beta value: 1.
model addParameter: #lambda value: [ :aModel| 
        (aModel atParameter: #beta) * 
        (aModel atCompartment: {#status->#I}) ].
model
   addTransitionFrom: '{#status: #S}'
   to: '{#status: #I}'
   probability: [ :m | m atParameter: #lambda ].
model 
   addTransitionFrom: '{#status: #I}' 
   to: '{#status: #R}' 
        probability: [ :m | m atParameter: #gamma ].
    #(#S #I #R) do: [ :each| 
        model 
            addTransitionFrom: {#status->each} 
            to: #empty 
            probability: [ :m | m atParameter: #mu ].
        ].
model 
        addTransitionFrom: #empty 
        to: '{#status: #S}' 
        probability: [ :m | m atParameter: #mu ].

multiHostConcern := KEConcern new.
multiHostConcern 
   addAttribute: #species 
   value: #(#mosquito #reservoir1 #reservoir2).

model integrate: multiHostConcern.

model 
        atParameter: #mu
        assignValue: 
        [ :aModel| |c val| 
            c := aModel currentCompartment at: #species.
            c = #mosquito ifTrue: [ val := 12.17 ].
            c = #reservoir1 ifTrue: [ val := 0.05 ].
            c = #reservoir2 ifTrue: [ val := 0.05 ].
            val
        ].
model atParameter: #N assignValue: [ :aModel| |c| 
        c := aModel currentCompartment at: #species.
        aModel sizeOfPopulation: c
        ].
model addParameter: #rho value: [ :aModel| |c val|
        c := aModel currentCompartment at: #species.
        c = #mosquito ifTrue: [ val := #(0 0.02 0.02) ].
        c = #reservoir1 ifTrue: [ val := #(0.02 0 0) ].
        c = #reservoir2 ifTrue: [ val := #(0.02 0 0) ].
        val
        ].
model atParameter: #lambda assignValue: [ :aModel|
        ((aModel atParameter: #beta) * 
        (aModel atParameter: #rho) * 
        (aModel atCompartment: {#status->#I})) sum
        ].

    model atCompartment: { #status->#I. #species->#mosquito } put: 1 atOthersPut: 0.
    model atCompartment: { #status->#S. #species->#mosquito } put: 9999.
    model atCompartment: { #status->#S. #species->#reservoir1 } put: 1000.
    model atCompartment: { #status->#S. #species->#reservoir2 } put: 2000.

simulator := KESimulator new: #Gillespie from: 0.0 to: 0.5 step: 0.0027.

simulator executeOn: model.
db := (KEDiagramBuilder new) data: ((simulator timeSeriesAt: '{#status: #I}') sqrt).
db open.

^ simulator -> model! !

!TestMosquitoScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 18:27'!
createModels

	beforePair := self before.
	beforeSimulator := beforePair key.
	beforeModel := beforePair value.
	self after.
	afterSimulator := Simulation withId: 'MosquitoGillepsie'.
	afterModel := Model withId: 'Mosquito'.
	(Visualization withId: 'MosquitoDiagramViz') open.	! !

!TestMosquitoScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 18:27'!
setUp

	KendrickInternalDSLEntities clearAll.
	"KEGillespieSimulatorForTesting newSeed." "stochastic but reproducible each time"
	! !

!TestMosquitoScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/6/2016 16:50'!
tearDown

	KendrickInternalDSLEntities clearAll.! !

!TestMosquitoScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 18:28'!
testEquivalence

	self createModels.

	beforeModel inspect.
	afterModel keModel inspect.

	"self assert: 
		(beforeSimulator allTimeSeries collect: [ :ts | ts peakOfEpidemic  ]) = 
		(afterSimulator allTimeSeries collect: [ :ts | ts peakOfEpidemic  ])"! !

TestCase subclass: #TestOneSpeciesSpatialScripts
	instanceVariableNames: 'beforeSimulator afterSimulator beforeModel afterModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestOneSpeciesSpatialScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/19/2016 18:02'!
after

Model OneSpeciesSpatial	
	population: 5000.

Concern Spatial
	attribute: #(patch -> 1  2  3  4  5);
	parameters: #(rho);
	rho: 0.03;
	transitions: #(
		1 -- rho --> 2.
		2 -- rho --> 3.
		3 -- rho --> 4.
		4 -- rho --> 5.
		5 -- rho --> 1.
	).
		
Concern Epidemiological
	attribute: #(status -> S  E  I  R);
	parameters: #(beta lambda sigma gamma v mu);
	v: 0.00274;
	mu: 0.0000365;
	sigma: 0.5;
	gamma: 0.25;
	lambda: #(beta*I/N);
	transitions: #(
		S -- lambda --> E.
		E -- sigma --> I.
		I -- gamma --> R.
		R -- v --> S.
		status -- mu --> Empty.
		Empty -- mu --> S.
	).
		
Composition OneSpatialCombo
	model: 'OneSpeciesSpatial';
	concern: 'Spatial';
	concern: 'Epidemiological';
	beta_patch: #(0.75 0.5 0.5 0.5 0.5);
	N: #(patch);
	lambda: #(beta*I_patch/N); "check how this should work"
	S_patch: #(900 1000 1000 1000 1000);
	I_patch: #(100 0 0 0 0).
		
Simulation SpatialRungeKutta rungeKutta
	for: 'OneSpeciesSpatial';
	from: 0; 
	to: 500; 
	step: 1.
	
Visualization SpatialDiagramViz diagram 
	for: 'SpatialRungeKutta';
	data: #(I_patch);
	legends: {'patch 1' . 'patch 2' . 'patch 3' . 'patch 4' . 'patch 5'};
	legendTitle: 'Infectious';
	xLabel: 'Time (days)'.! !

!TestOneSpeciesSpatialScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/19/2016 17:19'!
before

| spatialConcern seirsConcern model simulator db |
spatialConcern := KEConcern new.
 spatialConcern addAttribute: #patch value: (1 to: 5) asArray.
 spatialConcern addParameter: #rho.
 (1 to: 5) do: [ :i| 
     (i < 5) 
     ifTrue: [  
         spatialConcern 
             addTransitionFrom: { #patch->i } 
             to: { #patch->(i+1) } 
             probability: [ :aModel| aModel atParameter: #rho ].
     ]
     ifFalse: [  
         spatialConcern 
             addTransitionFrom: { #patch->i } 
             to: { #patch->1 } 
             probability: [ :aModel| aModel atParameter: #rho ].
     ]
 ]. 

seirsConcern := KEConcern new.
 seirsConcern attributes: {#status->#(#S #E #I #R)}.
 seirsConcern addParameters: { #beta. #gamma. #mu. #sigma. #v }.
 seirsConcern addParameter: #lambda value: [ :aModel | 
         (aModel atParameter: #beta) * 
         (aModel atCompartment: {#status->#I}) / (aModel atParameter: #N) ].
 seirsConcern
     addTransitionFrom: '{#status: #S}'
     to: '{#status: #E}'
     probability: [ :aModel | 
         (aModel atParameter: #lambda) ].
 seirsConcern 
     addTransitionFrom: '{#status: #E}' 
     to: '{#status: #I}' 
     probability: [ :aModel | aModel atParameter: #sigma ].
 seirsConcern 
     addTransitionFrom: '{#status: #I}' 
     to: '{#status: #R}' 
     probability: [ :aModel | aModel atParameter: #gamma ].
 seirsConcern 
     addTransitionFrom: '{#status: #R}' 
     to: '{#status: #S}' 
     probability: [ :aModel | aModel atParameter: #v ].
 #(S E I R) do: [ :each| 
     seirsConcern 
         addTransitionFrom: {#status->each}
         to: #empty 
         probability: [ :aModel | aModel atParameter: #mu ].
     ].
 seirsConcern 
     addTransitionFrom: #empty 
     to: '{#status: #S}' 
     probability: [ :aModel | aModel atParameter: #mu ].

     model := KEModel new.
 model population: (KEPopulation size: 5000).
 model integrate: spatialConcern.
 model integrate: seirsConcern.

model atParameter: #beta assignValue: [ :aModel| |c val| 
     c := aModel currentCompartment at: #patch.
     c = 1 ifTrue: [ val := 0.75 ] ifFalse: [ val := 0.5 ].
     val 
 ].
 model atParameter: #v assignValue: 0.00274.
 model atParameter: #mu assignValue: 0.0000365.
 model atParameter: #sigma assignValue: 0.5.
 model atParameter: #gamma assignValue: 0.25.
 model atParameter: #N assignValue: [ :aModel| |c| 
     c := aModel currentCompartment at: #patch.
     aModel sizeOfPopulation: c
     ].
 model atParameter: #rho assignValue: 0.03. 
 model atParameter: #lambda assignValue: [ :aModel| |c|
     c := aModel currentCompartment at: #patch.
     (aModel atParameter: #beta) * (aModel atCompartment: {#status->#I. #patch->c}) / (aModel atParameter: #N)
 ].

model atCompartment: { #status->#S. #patch->1 } put: 900 atOthersPut: 0.
 model atCompartment: { #status->#I. #patch->1 } put: 100.
 model atCompartment: { #status->#S. #patch->2 } put: 1000.
 model atCompartment: { #status->#S. #patch->3 } put: 1000.
 model atCompartment: { #status->#S. #patch->4 } put: 1000.
 model atCompartment: { #status->#S. #patch->5 } put: 1000.

simulator := KESimulator new: #RungeKutta from: 0 to: 500 step: 1.
 simulator executeOn: model.
 db := KEDiagramBuilder new.
 db data: [|tmp| tmp := OrderedCollection new.
 tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:1}').
 tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:2}').
 tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:3}').
 tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:4}').
 tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:5}').
 tmp] value.
 db legends: { 'patch 1'. 'patch 2'. 'patch 3'. 'patch 4'. 'patch 5'}.
 db legendTitle: 'Infectious'.
 db xLabel: 'Time (days)'.
 db open.

	^ simulator -> model! !

!TestOneSpeciesSpatialScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/19/2016 17:20'!
createModels

	| beforePair |
	beforePair := self before.
	beforeSimulator := beforePair key.
	beforeModel := beforePair value.
	self after.
	afterSimulator := Simulation withId: 'SpatialRungeKutta'.
	afterModel := Model withId: 'OneSpeciesSpatial'.
	(Visualization withId: 'SpatialDiagramViz') open.	! !

!TestOneSpeciesSpatialScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/19/2016 17:18'!
setUp

	KendrickInternalDSLEntities clearAll.
	"KEGillespieSimulatorForTesting newSeed." "stochastic but reproducible each time"! !

!TestOneSpeciesSpatialScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/6/2016 16:58'!
tearDown

	KendrickInternalDSLEntities clearAll.! !

!TestOneSpeciesSpatialScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/19/2016 17:18'!
testEquivalence

	self createModels.

	beforeModel inspect.
	afterModel keModel inspect.

	"self assert: 
		(beforeSimulator allTimeSeries collect: [ :ts | ts peakOfEpidemic  ]) = 
		(afterSimulator allTimeSeries collect: [ :ts | ts peakOfEpidemic  ])"! !

!KEModel methodsFor: '*ExtentionsForKendrick' stamp: 'NickPapoulias 4/13/2016 17:21'!
resetTime

	t := 0! !

Error subclass: #KendrickDSLSyntaxError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

KEGillespieSimulator subclass: #KEGillespieSimulatorForTesting
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

KEGillespieSimulatorForTesting class
	instanceVariableNames: 'seed'!

KEGillespieSimulatorForTesting class
	instanceVariableNames: 'seed'!

!KEGillespieSimulatorForTesting class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/13/2016 16:31'!
newSeed

	seed :=  (Random new next * 1000000000) asInteger -> (Random new next * 1000000000) asInteger! !

!KEGillespieSimulatorForTesting class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/12/2016 16:52'!
randPair

	^ (Random new seed: seed key) -> (Random new seed: seed value)! !

!KEGillespieSimulatorForTesting class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/12/2016 16:13'!
seed

	^ seed! !

!KEGillespieSimulatorForTesting class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/12/2016 17:46'!
seed: aPair

	seed := aPair! !

!KEGillespieSimulatorForTesting methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/12/2016 17:37'!
executeOn: aModel
	| rand1 rand2 t previousT randPair |
	super executeOn: aModel.
	t := tMin.
	previousT := t.
	randPair := self class randPair. "new generators with same seed (re-new controlled from class side)"
	rand1 := randPair key.
	rand2 := randPair value.
	self saveCompartmentsValueAtTime: tMin.

	[ t < tMax ]
		whileTrue: [ 
			| deltaT r1 r2 |
			r1 := rand1 next.
			r2 := rand2 next.
			deltaT := self doGillespieIteration: t randoms: (Array with: r1 with: r2).
			(deltaT = 0.0)
				ifTrue: [ ^ self ].
			t := t + deltaT.
			"Only store in the result from step to step"
			t > (previousT + step)
				ifTrue: [ 
					self saveCompartmentsValueAtTime: t.
					previousT := t ] ]! !

Object subclass: #KendrickEquations
	instanceVariableNames: 'eqAsSymExpr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!KendrickEquations methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 13:05'!
convertToCompatibleBackendRepresentation

	^ ((eqAsSymExpr asString
			copyReplaceAll: '#' with: '' asTokens: false)
			trimBoth: [ :c | (c = $() | (c = $)) ]) 
			findTokens: '''.'''! !

!KendrickEquations methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/4/2016 17:57'!
with: symExpression

	eqAsSymExpr := symExpression ! !

Object subclass: #KendrickInternalDSLEntities
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

KendrickInternalDSLEntities class
	instanceVariableNames: 'storedInstances'!

KendrickInternalDSLEntities class
	instanceVariableNames: 'storedInstances'!

KendrickInternalDSLEntities subclass: #Kendrick
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

Kendrick subclass: #Composition
	instanceVariableNames: 'model concerns compartments parameters keModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/11/2016 18:08'!
addCompositeAtrributeOrParameter: variable value: value

	|variablePlusAttributes|
	
	variablePlusAttributes := variable asString findTokens: '_'.
	"assume single attribute for the time being"
	
	(model attributeValuesIncludes: variablePlusAttributes first asSymbol)
		ifTrue: [ 
			self addCompositeAttribute: variablePlusAttributes value: value. 
		] ifFalse: [  
		(model parameters keys includes: variablePlusAttributes first asSymbol)
			ifTrue: [  
				self addCompositeParameter: variablePlusAttributes value: value.
			] ifFalse: [ 
					KendrickDSLSyntaxError 
								signal: 'Undefined Variable Being Set: ' , variable asString ]. 
		].
	! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/11/2016 17:49'!
addCompositeAttribute: variablePlusAttributes value: value

	compartments at: variablePlusAttributes put: value.
	(self allCompositeReferencesFor: variablePlusAttributes)
		doWithIndex: [:ref :index|  
			keModel atCompartment: ref put: (value at: index).		
		]! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/12/2016 18:08'!
addCompositeParameter: variablePlusAttributes value: value

	|entity|

	entity := ((concerns copy add: model; yourself) select: [ :e | e attributes keys 
		includes: (variablePlusAttributes last)]) first.

	parameters at: variablePlusAttributes put: value.
	keModel 
        addParameter: variablePlusAttributes first asSymbol
        value: 
        [ :aModel| |c index| 
            c := aModel currentCompartment at: variablePlusAttributes last.
				index := (entity attributes at: variablePlusAttributes last) indexOf: c.            
				value at: index
        ].

! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/11/2016 17:55'!
allCompositeReferencesFor: variablePlusAttributes

	|attribute compartment entity|
	compartment := variablePlusAttributes first.
	attribute := variablePlusAttributes last.
	entity := ((concerns copy add: model; yourself) select: [ :e | e attributes keys 
		includes: (variablePlusAttributes last)]) first.
	^ (entity attributes at: variablePlusAttributes last) 
			collect: [:comp | 
				{self referencesFor: compartment . (variablePlusAttributes last asSymbol) -> comp asSymbol}
			]
	! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/19/2016 18:12'!
concern: aString
	|concern|
	concern := Concern withId: aString.
	concerns add: concern.
	model integrate: concern.
	! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/12/2016 10:53'!
doesNotUnderstand: aMessage

	|variable value|

	"TODO: should also search in the concerns for setting comp/attri"
	
	variable := aMessage selector.
	variable := (variable asString trimRight: [: c | c =  $:]) asSymbol.
	value := aMessage arguments first.
 
	(model attributeValuesIncludes: variable)
		ifTrue: [ 
			compartments add: {variable -> value}.
			model atCompartment: variable value: value. 
		] ifFalse: [  
		(model parameters keys includes: variable)
			ifTrue: [  
				parameters at: variable put: value.
				model atParameter: variable value: value. "can be array here treat differently"
			] ifFalse: [ 
					(aMessage selector asString includes: $_)
						ifTrue: [ self addCompositeAtrributeOrParameter: variable value: value ]
						ifFalse: [  
							KendrickDSLSyntaxError 
								signal: 'Undefined Variable Being Set: ' , variable asString
						] 
					]. 
		].! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/11/2016 17:21'!
initialize

	concerns := OrderedCollection new. 
	compartments := Dictionary new. 
	parameters := Dictionary new.
	! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 17:41'!
model: aString

	model := Model withId: aString.
	keModel := model keModel ! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/11/2016 17:59'!
others: anInteger

	^ model others: anInteger. "TODO: propagate to concerns as well"

	! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/11/2016 17:55'!
referencesFor: anAttribute

	|assoc|

	assoc := (model attributes 
		associations select: [ :as | as value includes: anAttribute ]) first.
	^ assoc key asSymbol -> anAttribute asSymbol 

	! !

Kendrick subclass: #Concern
	instanceVariableNames: 'population attributes parameters equations explicitelySetCompartments otherCompartmentValue transitions keConcern'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 17:36'!
allTransitionRefsForAttribute: attribute
	
	^ (attributes at: attribute) collect: [:comp | '{#' , attribute asString , ': #' , comp , '}']! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 18:02'!
allTransitionsFor: aFromToList

		|from to propability|

		from := aFromToList first.
		propability := aFromToList second.
		to := aFromToList last.
		(self isAttribute: from)
			ifTrue: [ 
				(self allTransitionRefsForAttribute: from) do: [:fromRef |  
					keConcern 
   						addTransitionFrom: fromRef
   						to: (self transitionRefFor: to)
   						probability: [ :m | m atParameter: propability ].			
				]
			]
			ifFalse: [ "to = all"
				(self isAttribute: to)
					ifTrue: [  
						(self allTransitionRefsForAttribute: from) do: [:toRef |  
							keConcern
   								addTransitionFrom: (self transitionRefFor: from)
   								to: toRef
   								probability: [ :m | m atParameter: propability ].			
						]
					] ifFalse: [ "no attributes"  
						keConcern 
   								addTransitionFrom: (self transitionRefFor: from)
   								to: (self transitionRefFor: to)
   								probability: [ :m | m atParameter: propability ].
					]		
			 ] 
			
	! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 17:52'!
attribute: anArray

	attributes 
		at: anArray first 
		put: ((anArray allButFirst select: [:var | var ~= #->]) collect: [ :var | var asString asSymbol]).
	keConcern 
		addAttribute: anArray first 
		value: ((anArray allButFirst select: [:var | var ~= #->]) collect: [ :var | var asString asSymbol]). 
	! !

!Concern methodsFor: 'protected' stamp: 'NickPapoulias 4/11/2016 11:07'!
attributeValuesIncludes: variable

	^ (attributes values select: [ :values | values includes: variable ]) isEmpty not! !

!Concern methodsFor: 'protected' stamp: 'NickPapoulias 4/11/2016 17:48'!
attributes

	^ attributes
	! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 18:29'!
doesNotUnderstand: aMessage

	|variable value|
	
	variable := aMessage selector.
	variable := (variable asString trimRight: [: c | c =  $:]) asSymbol.
	value := aMessage arguments first.
 
	(self attributeValuesIncludes: variable)
		ifTrue: [ 
			explicitelySetCompartments add: {variable->value}.
		] ifFalse: [  
		(parameters keys includes: variable)
			ifTrue: [  
				parameters at: variable put: value.
			] ifFalse: [ 
					KendrickDSLSyntaxError 
								signal: 'Undefined Variable Being Set: ' , variable asString ]. 
		].
	
	! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 4/11/2016 11:57'!
equations: anArrayOfSymExpressions

	equations :=
		KendrickEquations new with: anArrayOfSymExpressions.
! !

!Concern methodsFor: 'protected' stamp: 'NickPapoulias 4/11/2016 17:24'!
initialize

	attributes := Dictionary new.
	parameters := Dictionary new.
	explicitelySetCompartments := OrderedCollection new.
	keConcern := KEConcern new.
! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 17:35'!
isAttribute: attribute
	
	^ (attributes keys includes: attribute) ! !

!Concern methodsFor: 'protected' stamp: 'NickPapoulias 4/11/2016 17:25'!
keConcern

	^ keConcern.
! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 4/11/2016 11:59'!
others: anInteger

	otherCompartmentValue := anInteger.! !

!Concern methodsFor: 'protected' stamp: 'NickPapoulias 4/11/2016 17:48'!
parameters

	^ parameters ! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 4/11/2016 11:07'!
parameters: anArray

	anArray do: [ :var | parameters at: var put: 0 ]! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 4/11/2016 11:58'!
population: anInteger

	population := anInteger.! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 17:37'!
transitionRefFor: aCompartment
	
	|assoc|

	
	aCompartment = 'Empty'
		ifTrue: [ ^ #empty ].
	assoc := (attributes associations select: [ :pair | pair value includes: aCompartment ]) first.
	^ '{#' , assoc key asString , ': #' , aCompartment , '}'.! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 17:55'!
transitions: aSymExpression
 
	transitions := KendrickTransitions new with: aSymExpression.
	transitions convertToCompatibleBackendRepresentation do: [:assoc |
		|from to propability|
		from := assoc key key asString asSymbol.
		propability := assoc key value.
		to := assoc value asString asSymbol.
		self allTransitionsFor: { from . propability . to } ].
		! !

!Kendrick class methodsFor: 'public-api' stamp: 'NickPapoulias 4/18/2016 17:43'!
composition

	^ Composition new! !

!Kendrick class methodsFor: 'public-api' stamp: 'NickPapoulias 4/18/2016 17:43'!
concern

	^ Concern new! !

!Kendrick class methodsFor: 'public-api' stamp: 'NickPapoulias 4/18/2016 17:50'!
doesNotUnderstand: aMessage

	^ self new id: aMessage selector! !

!Kendrick class methodsFor: 'public-api' stamp: 'NickPapoulias 4/18/2016 17:41'!
model

	^ Model new! !

Kendrick subclass: #Model
	instanceVariableNames: 'population attributes parameters equations keModel explicitelySetCompartments otherCompartmentValue transitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/11/2016 15:12'!
addParameter: variable value: value

	value isNumber 
		ifTrue: [keModel addParameter: variable value: value]
		ifFalse: [ "array" 
			keModel 
				addParameter: variable 
				value: (self covertSymbolicExpressionToBlock: value).
		]! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/12/2016 17:21'!
allTransitionRefsForAttribute: attribute
	
	^ (attributes at: attribute) collect: [:comp | '{#' , attribute asString , ': #' , comp , '}']! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/12/2016 14:35'!
allTransitionRefsForAttributeWith: aCompartment
	
	|assoc|

	aCompartment asString translateToLowercase = 'empty'
		ifTrue: [ ^ { #empty } ].
	
	assoc := (attributes associations select: [ :pair | pair value includes: aCompartment ]).
	^ assoc value collect: [:comp | '{#' , assoc key asString , ': #' , comp , '}']! !

!Model methodsFor: 'public-api' stamp: 'NickPapoulias 4/12/2016 17:32'!
allTransitionsFor: aFromToList

		|from to propability|

		from := aFromToList first.
		propability := aFromToList second.
		to := aFromToList last.
		(self isAttribute: from)
			ifTrue: [ 
				(self allTransitionRefsForAttribute: from) do: [:fromRef |  
					keModel
   						addTransitionFrom: fromRef
   						to: (self transitionRefFor: to)
   						probability: [ :m | m atParameter: propability ].			
				]
			]
			ifFalse: [ "to = all"
				(self isAttribute: to)
					ifTrue: [  
						(self allTransitionRefsForAttribute: from) do: [:toRef |  
							keModel
   								addTransitionFrom: (self transitionRefFor: from)
   								to: toRef
   								probability: [ :m | m atParameter: propability ].			
						]
					] ifFalse: [ "no attributes"  
						keModel
   								addTransitionFrom: (self transitionRefFor: from)
   								to: (self transitionRefFor: to)
   								probability: [ :m | m atParameter: propability ].
					]		
			 ] 
			
	! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/11/2016 18:03'!
atCompartment: variable value: value

	|assoc|

	assoc := (attributes associations select: [ :as | as value includes: variable ]) first.
	keModel atCompartment: {assoc key -> variable} put: value
	! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/12/2016 10:53'!
atParameter: variable value: value

	value isNumber 
		ifTrue: [keModel atParameter: variable assignValue: value]
		ifFalse: [ "array" 
			keModel 
				atParameter: variable 
				assignValue: (self covertSymbolicExpressionToBlock: value).
		]! !

!Model methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 17:52'!
attribute: anArray

	attributes 
		at: anArray first 
		put: ((anArray allButFirst select: [:var | var ~= #->]) collect: [ :var | var asString asSymbol]).
	keModel 
		addAttribute: anArray first 
		value: ((anArray allButFirst select: [:var | var ~= #->]) collect: [ :var | var asString asSymbol]). 
	! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/19/2016 18:42'!
attributeOrParameter: aName


	^ (parameters keys includes: aName)
		ifTrue: [ keModel atParameter: aName ]
		ifFalse: [  
			|assoc|
			assoc := (attributes associations select: [ :as | as value includes: aName ]).
			assoc isEmpty 
				ifTrue: [ 
					(aName asString includes: $_)
							ifTrue: [ |tokens c|
								tokens := aName findTokens: '_'.
								c := keModel currentCompartment at: tokens second.
								assoc := (attributes associations select: [ :as | as value includes: tokens first ]).
								keModel atCompartment: { assoc first key -> tokens first asSymbol . tokens second -> c } 
							]
							ifFalse: [keModel sizeOfPopulation: (keModel currentCompartment at: aName asSymbol)] 
				] "assume attribute name"
				ifFalse: [keModel atCompartment: { assoc first key -> aName }]
		
		]
	! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/4/2016 17:28'!
attributeValuesIncludes: variable

	^ (attributes values select: [ :values | values includes: variable ]) isEmpty not! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/11/2016 17:30'!
attributes

	^ attributes! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/19/2016 15:50'!
covertSymbolicExpressionToBlock: anArray

			|body|

			body := (anArray collect: [ :token | 
				(#(- + * / sum sqrt) includes: token)
					ifTrue: [ token asString ]
					ifFalse: [ 
						 '(self attributeOrParameter: ' ,  '#' , token asString , ')' 
					]			
			]) inject: '' into: [ :expr :token | 
				(#(sum sqrt size) includes: token)
					ifFalse: [expr , ' ' , token]
					ifTrue: [ '(' , expr , ')' , token ]
			].
			"the argument is superficial here since it is captured (backward compatibility)"
			^ Compiler new		evaluate: '[:aModel |' , body , ']'
					in: thisContext sender
					to: thisContext sender receiver
					notifying: nil
					ifFail: [^nil]
					logged: false.! !

!Model methodsFor: 'public-api' stamp: 'NickPapoulias 4/11/2016 14:32'!
doesNotUnderstand: aMessage

	|variable value|
	
	variable := aMessage selector.
	variable := (variable asString trimRight: [: c | c =  $:]) asSymbol.
	value := aMessage arguments first.
 
	(self attributeValuesIncludes: variable)
		ifTrue: [ 
			explicitelySetCompartments add: variable.
			self atCompartment: variable value: value. 
		] ifFalse: [  
		(parameters keys includes: variable)
			ifTrue: [  
				parameters at: variable put: value.
				self addParameter: variable value: value. "can be array here treat differently"
			] ifFalse: [ 
					KendrickDSLSyntaxError 
								signal: 'Undefined Variable Being Set: ' , variable asString ]. 
		].
	
	! !

!Model methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 13:06'!
equations: anArrayOfSymExpressions

	equations :=
		KendrickEquations new with: anArrayOfSymExpressions.
	keModel addEquations: equations convertToCompatibleBackendRepresentation.
	! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/12/2016 10:49'!
initialize

	keModel := KEModel new.
	attributes := Dictionary new.
	parameters := Dictionary new.
	parameters at: #N put: nil. "seems to be a default one"
	explicitelySetCompartments := OrderedCollection new.! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/19/2016 18:32'!
integrate: concern.

	keModel integrate: concern keConcern.
	parameters addAll: concern parameters.
	concern parameters keysAndValuesDo: [ :key :value | self addParameter: key value: value ].
	attributes addAll: concern attributes. "init compartments from concerns not possible just add"
	
! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/12/2016 17:21'!
isAttribute: attribute
	
	^ (attributes keys includes: attribute) ! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/5/2016 16:27'!
keModel

	^ keModel! !

!Model methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 16:05'!
others: anInteger

	otherCompartmentValue := anInteger.

	keModel attributes
		associationsDo: [ :assoc | 
			assoc value do: [:var |
				(explicitelySetCompartments includes: var)
					ifFalse: [ self atCompartment: var value: anInteger ]
			] 
		]! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/11/2016 17:30'!
parameters

	^ parameters ! !

!Model methodsFor: 'public-api' stamp: 'NickPapoulias 4/18/2016 18:55'!
parameters: anArray

	anArray do: [ :var | parameters at: var put: 0 ]! !

!Model methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:51'!
population: anInteger

	population := anInteger.
	keModel population: (KEPopulation size: anInteger).! !

!Model methodsFor: 'protected' stamp: 'NickPapoulias 4/19/2016 17:37'!
transitionRefFor: aCompartment
	
	|assoc|

	
	aCompartment = 'Empty'
		ifTrue: [ ^ #empty ].
	assoc := (attributes associations select: [ :pair | pair value includes: aCompartment ]) first.
	^ '{#' , assoc key asString , ': #' , aCompartment , '}'.! !

!Model methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 17:55'!
transitions: aSymExpression

	transitions :=
		KendrickTransitions new with: aSymExpression.
	transitions convertToCompatibleBackendRepresentation do: [:assoc |
		|from to propability|
		from := assoc key key asString asSymbol.
		propability := assoc key value.
		to := assoc value asString asSymbol.
		self allTransitionsFor: { from . propability . to } ].
		! !

Kendrick subclass: #Simulation
	instanceVariableNames: 'algo model from to step simulator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!Simulation methodsFor: 'protected' stamp: 'NickPapoulias 4/5/2016 15:38'!
allTimeSeries

	^ simulator allTimeSeries ! !

!Simulation methodsFor: 'protected' stamp: 'NickPapoulias 4/11/2016 18:15'!
execute

	simulator executeOn: model keModel.! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/18/2016 17:41'!
for: aModelID

	model := Model withId: aModelID.! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:27'!
from: aNumber

	from := aNumber! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/18/2016 17:51'!
gillespie

	^ self  
		with: #Gillespie! !

!Simulation methodsFor: 'protected' stamp: 'NickPapoulias 4/12/2016 16:33'!
init

	algo = #testGillespie
		ifTrue: [ simulator := (KEGillespieSimulatorForTesting new). 
							simulator tMin: from; tMax: to; step: step ]
		ifFalse: [  simulator := KESimulator new: algo from: from to: to step: step ]! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/11/2016 18:10'!
model

	^ model! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/18/2016 17:51'!
rungeKutta

	^ self  
		with: #RungeKutta! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:29'!
step: aNumber

	step := aNumber! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/18/2016 17:50'!
testGillespie

	^ self  
		with: #testGillespie! !

!Simulation methodsFor: 'protected' stamp: 'NickPapoulias 4/11/2016 18:13'!
timeSeriesAt: anIdentifier

	^ simulator timeSeriesAt: anIdentifier ! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:27'!
to: aNumber

	to := aNumber! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:24'!
with: anAlgorithm

	algo := anAlgorithm ! !

Kendrick subclass: #Visualization
	instanceVariableNames: 'simulation xLabel diag builder explicitDataSet lazyDataBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 18:39'!
data: anArray "should be full math-expression"

	"refactor !!!!"

	explicitDataSet := true.

	anArray size = 2 ifTrue: [  
		lazyDataBlock := [
			diag data: 
				((simulation 
					timeSeriesAt: 
						(simulation model transitionRefFor: anArray first))
					perform: anArray last)
			]
		] ifFalse: [  
			lazyDataBlock := [
				|name|
				name := anArray first.
				(name asString includes: $_)
					ifTrue: [ |tokens tSeries| 
						tokens := name findTokens: '_'.
						tSeries := OrderedCollection new.
						(simulation model allTransitionRefsForAttribute: tokens second) do: [:comp |  
							|compositeCompartment|
							compositeCompartment 
								:= (simulation model transitionRefFor: anArray first) allButLast.
							compositeCompartment := compositeCompartment , comp allButFirst.
							tSeries add: (simulation timeSeriesAt: compositeCompartment).				
						].
						diag data: tSeries.
					]
					ifFalse: [
						diag data: 
							(simulation 
								timeSeriesAt: 
									(simulation model transitionRefFor: anArray first))  ]
					
			]
		]! !

!Visualization methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 17:58'!
diagram

	^ self 
		with: KEDiagramBuilder! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/11/2016 18:15'!
for: simulationID

	simulation := (Simulation withId: simulationID) init.! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/11/2016 18:16'!
initialize

	explicitDataSet := false! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 18:19'!
legendTitle: aString

	diag legendTitle: aString! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 18:19'!
legends: anOrderedCollection

	diag legends: anOrderedCollection ! !

!Visualization methodsFor: 'protected' stamp: 'NickPapoulias 4/12/2016 09:35'!
open

	simulation execute.
	explicitDataSet 
		ifFalse: [ diag data: simulation allTimeSeries ]
		ifTrue: [ lazyDataBlock value ].
	diag xLabel: xLabel.
	diag open! !

!Visualization methodsFor: 'protected' stamp: 'NickPapoulias 4/12/2016 09:35'!
with: aBuilder

	builder := aBuilder.
	diag := builder new.! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:36'!
xLabel: aString

	xLabel := aString! !

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:23'!
clearAll

	self allSubclassesDo: [ :subclass | subclass clearAllInstances ]! !

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:20'!
clearAllInstances

	storedInstances := nil! !

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 17:59'!
doesNotUnderstand: aMessage

	^ self new id: aMessage selector! !

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:22'!
new

	^ self storedInstances addFirst: self basicNew initialize! !

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:20'!
storedInstances

	^ storedInstances ifNil: [ storedInstances := OrderedCollection new ]! !

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:19'!
withId: anId

	^ (self storedInstances select: [ :instance | instance id = anId ]) first.! !

!KendrickInternalDSLEntities methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/11/2016 16:31'!
doesNotUnderstand: aMessage

	KendrickDSLSyntaxError signal: self class name , ' has no keyword ' , aMessage selector.! !

!KendrickInternalDSLEntities methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/4/2016 16:51'!
id

	^ id! !

!KendrickInternalDSLEntities methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/4/2016 16:52'!
id: aString

	id := aString! !

Object subclass: #KendrickTransitions
	instanceVariableNames: 'eqAsSymExpr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!KendrickTransitions methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 19:19'!
convertToCompatibleBackendRepresentation

	|transitions currentTransition|

	transitions := OrderedCollection new.
	currentTransition := nil.

	eqAsSymExpr  do: [ :token | 
		(token = #--> or: [ token = #-- ])
			ifTrue: [  
				"do nothing"
			] ifFalse: [ 
				token = #'.'
					ifTrue: [ transitions add: currentTransition. currentTransition := nil ]
					ifFalse: [ 
						currentTransition = nil 
							ifTrue: [ currentTransition := token ]
							ifFalse: [ currentTransition := currentTransition -> token ]  
					]
			]
	].

	^ transitions 

	! !

!KendrickTransitions methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/11/2016 11:30'!
with: symExpression

	eqAsSymExpr := symExpression ! !
